# This file was auto-generated by Fern from our API Definition.

import typing
import urllib.parse
from json.decoder import JSONDecodeError

import pydantic
import typing_extensions

from ...core.api_error import ApiError
from ...core.client_wrapper import AsyncClientWrapper, SyncClientWrapper
from ...core.jsonable_encoder import jsonable_encoder
from ...core.remove_none_from_dict import remove_none_from_dict
from ...types.get_list_list_response_collection_compound_document import GetListListResponseCollectionCompoundDocument
from ...types.get_list_profiles_request_fields_profile_item import GetListProfilesRequestFieldsProfileItem
from ...types.get_list_relationships_response_collection import GetListRelationshipsResponseCollection
from ...types.get_list_request_fields_list_item import GetListRequestFieldsListItem
from ...types.get_list_retrieve_response_compound_document import GetListRetrieveResponseCompoundDocument
from ...types.get_list_tag_relationship_list_response_collection import GetListTagRelationshipListResponseCollection
from ...types.get_lists_request_fields_list_item import GetListsRequestFieldsListItem
from ...types.get_profile_response_collection import GetProfileResponseCollection
from ...types.get_tag_response_collection import GetTagResponseCollection
from ...types.list_create_query_resource_object import ListCreateQueryResourceObject
from ...types.list_members_add_query_data_item import ListMembersAddQueryDataItem
from ...types.list_members_delete_query_data_item import ListMembersDeleteQueryDataItem
from ...types.list_partial_update_query_resource_object import ListPartialUpdateQueryResourceObject
from ...types.patch_list_partial_update_response import PatchListPartialUpdateResponse
from ...types.post_list_create_response import PostListCreateResponse

# this is used as the default value for optional parameters
OMIT = typing.cast(typing.Any, ...)


class ListsClient:
    def __init__(self, *, client_wrapper: SyncClientWrapper):
        self._client_wrapper = client_wrapper

    def get_lists(
        self,
        *,
        fields_list: typing.Union[
            typing.Optional[GetListsRequestFieldsListItem], typing.List[GetListsRequestFieldsListItem]
        ],
        fields_tag: typing.Union[
            typing.Optional[typing_extensions.Literal["name"]], typing.List[typing_extensions.Literal["name"]]
        ],
        filter: typing.Optional[str] = None,
        include: typing.Union[
            typing.Optional[typing_extensions.Literal["tags"]], typing.List[typing_extensions.Literal["tags"]]
        ],
        page_cursor: typing.Optional[str] = None,
    ) -> GetListListResponseCollectionCompoundDocument:
        """
        Get all lists in an account.

        Filter to request a subset of all lists. Lists can be filtered by `id`, `name`, `created`, and `updated` fields.

        Returns a maximum of 10 results per page.<br><br>*Rate limits*:<br>Burst: `75/s`<br>Steady: `700/m`

        **Scopes:**
        `list:read`

        Parameters:
            - fields_list: typing.Union[typing.Optional[GetListsRequestFieldsListItem], typing.List[GetListsRequestFieldsListItem]]. For more information please visit https://developers.klaviyo.com/en/v2023-08-15/reference/api-overview#sparse-fieldsets

            - fields_tag: typing.Union[typing.Optional[typing_extensions.Literal["name"]], typing.List[typing_extensions.Literal["name"]]]. For more information please visit https://developers.klaviyo.com/en/v2023-08-15/reference/api-overview#sparse-fieldsets

            - filter: typing.Optional[str]. For more information please visit https://developers.klaviyo.com/en/v2023-08-15/reference/api-overview#filtering<br>Allowed field(s)/operator(s):<br>`name`: `any`, `equals`<br>`id`: `any`, `equals`<br>`created`: `greater-than`<br>`updated`: `greater-than`

            - include: typing.Union[typing.Optional[typing_extensions.Literal["tags"]], typing.List[typing_extensions.Literal["tags"]]]. For more information please visit https://developers.klaviyo.com/en/v2023-08-15/reference/api-overview#relationships

            - page_cursor: typing.Optional[str]. For more information please visit https://developers.klaviyo.com/en/v2023-08-15/reference/api-overview#pagination
        """
        _response = self._client_wrapper.httpx_client.request(
            "GET",
            urllib.parse.urljoin(f"{self._client_wrapper.get_base_url()}/", "api/lists"),
            params=remove_none_from_dict(
                {
                    "fields[list]": fields_list,
                    "fields[tag]": fields_tag,
                    "filter": filter,
                    "include": include,
                    "page[cursor]": page_cursor,
                }
            ),
            headers=self._client_wrapper.get_headers(),
            timeout=60,
        )
        if 200 <= _response.status_code < 300:
            return pydantic.parse_obj_as(GetListListResponseCollectionCompoundDocument, _response.json())  # type: ignore
        try:
            _response_json = _response.json()
        except JSONDecodeError:
            raise ApiError(status_code=_response.status_code, body=_response.text)
        raise ApiError(status_code=_response.status_code, body=_response_json)

    def create_list(self, *, data: ListCreateQueryResourceObject) -> PostListCreateResponse:
        """
        Create a new list.<br><br>*Rate limits*:<br>Burst: `10/s`<br>Steady: `150/m`

        **Scopes:**
        `list:write`

        Parameters:
            - data: ListCreateQueryResourceObject.
        """
        _response = self._client_wrapper.httpx_client.request(
            "POST",
            urllib.parse.urljoin(f"{self._client_wrapper.get_base_url()}/", "api/lists"),
            json=jsonable_encoder({"data": data}),
            headers=self._client_wrapper.get_headers(),
            timeout=60,
        )
        if 200 <= _response.status_code < 300:
            return pydantic.parse_obj_as(PostListCreateResponse, _response.json())  # type: ignore
        try:
            _response_json = _response.json()
        except JSONDecodeError:
            raise ApiError(status_code=_response.status_code, body=_response.text)
        raise ApiError(status_code=_response.status_code, body=_response_json)

    def get_list(
        self,
        id: str,
        *,
        additional_fields_list: typing.Union[
            typing.Optional[typing_extensions.Literal["profile_count"]],
            typing.List[typing_extensions.Literal["profile_count"]],
        ],
        fields_list: typing.Union[
            typing.Optional[GetListRequestFieldsListItem], typing.List[GetListRequestFieldsListItem]
        ],
        fields_tag: typing.Union[
            typing.Optional[typing_extensions.Literal["name"]], typing.List[typing_extensions.Literal["name"]]
        ],
        include: typing.Union[
            typing.Optional[typing_extensions.Literal["tags"]], typing.List[typing_extensions.Literal["tags"]]
        ],
    ) -> GetListRetrieveResponseCompoundDocument:
        """
        Get a list with the given list ID.<br><br>*Rate limits*:<br>Burst: `75/s`<br>Steady: `700/m`<br><br>Rate limits when using the `additional-fields[list]=profile_count` parameter in your API request:<br>Burst: `1/s`<br>Steady: `15/m`<br><br>To learn more about how the `additional-fields` parameter impacts rate limits, check out our [Rate limits, status codes, and errors](https://developers.klaviyo.com/en/v2023-08-15/docs/rate_limits_and_error_handling) guide.

        **Scopes:**
        `list:read`

        Parameters:
            - id: str. Primary key that uniquely identifies this list. Generated by Klaviyo.

            - additional_fields_list: typing.Union[typing.Optional[typing_extensions.Literal["profile_count"]], typing.List[typing_extensions.Literal["profile_count"]]]. Request additional fields not included by default in the response. Supported values: 'profile_count'

            - fields_list: typing.Union[typing.Optional[GetListRequestFieldsListItem], typing.List[GetListRequestFieldsListItem]]. For more information please visit https://developers.klaviyo.com/en/v2023-08-15/reference/api-overview#sparse-fieldsets

            - fields_tag: typing.Union[typing.Optional[typing_extensions.Literal["name"]], typing.List[typing_extensions.Literal["name"]]]. For more information please visit https://developers.klaviyo.com/en/v2023-08-15/reference/api-overview#sparse-fieldsets

            - include: typing.Union[typing.Optional[typing_extensions.Literal["tags"]], typing.List[typing_extensions.Literal["tags"]]]. For more information please visit https://developers.klaviyo.com/en/v2023-08-15/reference/api-overview#relationships
        """
        _response = self._client_wrapper.httpx_client.request(
            "GET",
            urllib.parse.urljoin(f"{self._client_wrapper.get_base_url()}/", f"api/lists/{id}"),
            params=remove_none_from_dict(
                {
                    "additional-fields[list]": additional_fields_list,
                    "fields[list]": fields_list,
                    "fields[tag]": fields_tag,
                    "include": include,
                }
            ),
            headers=self._client_wrapper.get_headers(),
            timeout=60,
        )
        if 200 <= _response.status_code < 300:
            return pydantic.parse_obj_as(GetListRetrieveResponseCompoundDocument, _response.json())  # type: ignore
        try:
            _response_json = _response.json()
        except JSONDecodeError:
            raise ApiError(status_code=_response.status_code, body=_response.text)
        raise ApiError(status_code=_response.status_code, body=_response_json)

    def update_list(self, id: str, *, data: ListPartialUpdateQueryResourceObject) -> PatchListPartialUpdateResponse:
        """
        Update the name of a list with the given list ID.<br><br>*Rate limits*:<br>Burst: `10/s`<br>Steady: `150/m`

        **Scopes:**
        `list:write`

        Parameters:
            - id: str. Primary key that uniquely identifies this list. Generated by Klaviyo.

            - data: ListPartialUpdateQueryResourceObject.
        """
        _response = self._client_wrapper.httpx_client.request(
            "PATCH",
            urllib.parse.urljoin(f"{self._client_wrapper.get_base_url()}/", f"api/lists/{id}"),
            json=jsonable_encoder({"data": data}),
            headers=self._client_wrapper.get_headers(),
            timeout=60,
        )
        if 200 <= _response.status_code < 300:
            return pydantic.parse_obj_as(PatchListPartialUpdateResponse, _response.json())  # type: ignore
        try:
            _response_json = _response.json()
        except JSONDecodeError:
            raise ApiError(status_code=_response.status_code, body=_response.text)
        raise ApiError(status_code=_response.status_code, body=_response_json)

    def delete_list(self, id: str) -> None:
        """
        Delete a list with the given list ID.<br><br>*Rate limits*:<br>Burst: `10/s`<br>Steady: `150/m`

        **Scopes:**
        `list:write`

        Parameters:
            - id: str. Primary key that uniquely identifies this list. Generated by Klaviyo.
        """
        _response = self._client_wrapper.httpx_client.request(
            "DELETE",
            urllib.parse.urljoin(f"{self._client_wrapper.get_base_url()}/", f"api/lists/{id}"),
            headers=self._client_wrapper.get_headers(),
            timeout=60,
        )
        if 200 <= _response.status_code < 300:
            return
        try:
            _response_json = _response.json()
        except JSONDecodeError:
            raise ApiError(status_code=_response.status_code, body=_response.text)
        raise ApiError(status_code=_response.status_code, body=_response_json)

    def get_list_relationships_tags(self, id: str) -> GetListTagRelationshipListResponseCollection:
        """
        Returns the tag IDs of all tags associated with the given list.<br><br>*Rate limits*:<br>Burst: `3/s`<br>Steady: `60/m`

        **Scopes:**
        `list:read`
        `tags:read`

        Parameters:
            - id: str.
        """
        _response = self._client_wrapper.httpx_client.request(
            "GET",
            urllib.parse.urljoin(f"{self._client_wrapper.get_base_url()}/", f"api/lists/{id}/relationships/tags"),
            headers=self._client_wrapper.get_headers(),
            timeout=60,
        )
        if 200 <= _response.status_code < 300:
            return pydantic.parse_obj_as(GetListTagRelationshipListResponseCollection, _response.json())  # type: ignore
        try:
            _response_json = _response.json()
        except JSONDecodeError:
            raise ApiError(status_code=_response.status_code, body=_response.text)
        raise ApiError(status_code=_response.status_code, body=_response_json)

    def get_list_tags(
        self,
        id: str,
        *,
        fields_tag: typing.Union[
            typing.Optional[typing_extensions.Literal["name"]], typing.List[typing_extensions.Literal["name"]]
        ],
    ) -> GetTagResponseCollection:
        """
        Return all tags associated with the given list ID.<br><br>*Rate limits*:<br>Burst: `3/s`<br>Steady: `60/m`

        **Scopes:**
        `list:read`
        `tags:read`

        Parameters:
            - id: str.

            - fields_tag: typing.Union[typing.Optional[typing_extensions.Literal["name"]], typing.List[typing_extensions.Literal["name"]]]. For more information please visit https://developers.klaviyo.com/en/v2023-08-15/reference/api-overview#sparse-fieldsets
        """
        _response = self._client_wrapper.httpx_client.request(
            "GET",
            urllib.parse.urljoin(f"{self._client_wrapper.get_base_url()}/", f"api/lists/{id}/tags"),
            params=remove_none_from_dict({"fields[tag]": fields_tag}),
            headers=self._client_wrapper.get_headers(),
            timeout=60,
        )
        if 200 <= _response.status_code < 300:
            return pydantic.parse_obj_as(GetTagResponseCollection, _response.json())  # type: ignore
        try:
            _response_json = _response.json()
        except JSONDecodeError:
            raise ApiError(status_code=_response.status_code, body=_response.text)
        raise ApiError(status_code=_response.status_code, body=_response_json)

    def get_list_relationships_profiles(
        self, id: str, *, page_cursor: typing.Optional[str] = None
    ) -> GetListRelationshipsResponseCollection:
        """
        Get profile membership [relationships](https://developers.klaviyo.com/en/reference/api_overview#relationships) for a list with the given list ID.<br><br>*Rate limits*:<br>Burst: `75/s`<br>Steady: `700/m`

        **Scopes:**
        `list:read`
        `profiles:read`

        Parameters:
            - id: str.

            - page_cursor: typing.Optional[str]. For more information please visit https://developers.klaviyo.com/en/v2023-08-15/reference/api-overview#pagination
        """
        _response = self._client_wrapper.httpx_client.request(
            "GET",
            urllib.parse.urljoin(f"{self._client_wrapper.get_base_url()}/", f"api/lists/{id}/relationships/profiles"),
            params=remove_none_from_dict({"page[cursor]": page_cursor}),
            headers=self._client_wrapper.get_headers(),
            timeout=60,
        )
        if 200 <= _response.status_code < 300:
            return pydantic.parse_obj_as(GetListRelationshipsResponseCollection, _response.json())  # type: ignore
        try:
            _response_json = _response.json()
        except JSONDecodeError:
            raise ApiError(status_code=_response.status_code, body=_response.text)
        raise ApiError(status_code=_response.status_code, body=_response_json)

    def create_list_relationships(self, id: str, *, data: typing.List[ListMembersAddQueryDataItem]) -> None:
        """
        Add a profile to a list with the given list ID.

        It is recommended that you use the [Subscribe Profiles endpoint](https://developers.klaviyo.com/en/reference/subscribe_profiles) if you're trying to give a profile consent to receive email marketing.

        This endpoint accepts a maximum of 1000 profiles per call.<br><br>*Rate limits*:<br>Burst: `10/s`<br>Steady: `150/m`

        **Scopes:**
        `list:write`
        `profiles:write`

        Parameters:
            - id: str.

            - data: typing.List[ListMembersAddQueryDataItem].
        """
        _response = self._client_wrapper.httpx_client.request(
            "POST",
            urllib.parse.urljoin(f"{self._client_wrapper.get_base_url()}/", f"api/lists/{id}/relationships/profiles"),
            json=jsonable_encoder({"data": data}),
            headers=self._client_wrapper.get_headers(),
            timeout=60,
        )
        if 200 <= _response.status_code < 300:
            return
        try:
            _response_json = _response.json()
        except JSONDecodeError:
            raise ApiError(status_code=_response.status_code, body=_response.text)
        raise ApiError(status_code=_response.status_code, body=_response_json)

    def delete_list_relationships(self, id: str, *, data: typing.List[ListMembersDeleteQueryDataItem]) -> None:
        """
        Remove a profile from a list with the given list ID.

        The provided profile will no longer receive marketing from this particular list once removed.

        Removing a profile from a list will not impact the profile's consent status or subscription status in general.
        To update a profile's subscription status, please use the [Unsubscribe Profiles endpoint](https://developers.klaviyo.com/en/reference/unsubscribe_profiles).

        This endpoint accepts a maximum of 1000 profiles per call.<br><br>*Rate limits*:<br>Burst: `10/s`<br>Steady: `150/m`

        **Scopes:**
        `list:write`
        `profiles:write`

        Parameters:
            - id: str.

            - data: typing.List[ListMembersDeleteQueryDataItem].
        """
        _response = self._client_wrapper.httpx_client.request(
            "DELETE",
            urllib.parse.urljoin(f"{self._client_wrapper.get_base_url()}/", f"api/lists/{id}/relationships/profiles"),
            json=jsonable_encoder({"data": data}),
            headers=self._client_wrapper.get_headers(),
            timeout=60,
        )
        if 200 <= _response.status_code < 300:
            return
        try:
            _response_json = _response.json()
        except JSONDecodeError:
            raise ApiError(status_code=_response.status_code, body=_response.text)
        raise ApiError(status_code=_response.status_code, body=_response_json)

    def get_list_profiles(
        self,
        id: str,
        *,
        additional_fields_profile: typing.Union[
            typing.Optional[typing_extensions.Literal["predictive_analytics"]],
            typing.List[typing_extensions.Literal["predictive_analytics"]],
        ],
        fields_profile: typing.Union[
            typing.Optional[GetListProfilesRequestFieldsProfileItem],
            typing.List[GetListProfilesRequestFieldsProfileItem],
        ],
        filter: typing.Optional[str] = None,
        page_cursor: typing.Optional[str] = None,
        page_size: typing.Optional[int] = None,
    ) -> GetProfileResponseCollection:
        """
        Get all profiles within a list with the given list ID.

        Filter to request a subset of all profiles. Profiles can be filtered by `email`, `phone_number`, and `push_token` fields.<br><br>*Rate limits*:<br>Burst: `75/s`<br>Steady: `700/m`<br><br>Rate limits when using the `additional-fields[profile]=predictive_analytics` parameter in your API request:<br>Burst: `10/s`<br>Steady: `150/m`<br><br>To learn more about how the `additional-fields` parameter impacts rate limits, check out our [Rate limits, status codes, and errors](https://developers.klaviyo.com/en/v2023-08-15/docs/rate_limits_and_error_handling) guide.

        **Scopes:**
        `list:read`
        `profiles:read`

        Parameters:
            - id: str.

            - additional_fields_profile: typing.Union[typing.Optional[typing_extensions.Literal["predictive_analytics"]], typing.List[typing_extensions.Literal["predictive_analytics"]]]. Request additional fields not included by default in the response. Supported values: 'predictive_analytics'

            - fields_profile: typing.Union[typing.Optional[GetListProfilesRequestFieldsProfileItem], typing.List[GetListProfilesRequestFieldsProfileItem]]. For more information please visit https://developers.klaviyo.com/en/v2023-08-15/reference/api-overview#sparse-fieldsets

            - filter: typing.Optional[str]. For more information please visit https://developers.klaviyo.com/en/v2023-08-15/reference/api-overview#filtering<br>Allowed field(s)/operator(s):<br>`email`: `any`<br>`phone_number`: `any`<br>`push_token`: `any`<br>`_kx`: `equals`

            - page_cursor: typing.Optional[str]. For more information please visit https://developers.klaviyo.com/en/v2023-08-15/reference/api-overview#pagination

            - page_size: typing.Optional[int]. Default: 20. Min: 1. Max: 100.
        """
        _response = self._client_wrapper.httpx_client.request(
            "GET",
            urllib.parse.urljoin(f"{self._client_wrapper.get_base_url()}/", f"api/lists/{id}/profiles"),
            params=remove_none_from_dict(
                {
                    "additional-fields[profile]": additional_fields_profile,
                    "fields[profile]": fields_profile,
                    "filter": filter,
                    "page[cursor]": page_cursor,
                    "page[size]": page_size,
                }
            ),
            headers=self._client_wrapper.get_headers(),
            timeout=60,
        )
        if 200 <= _response.status_code < 300:
            return pydantic.parse_obj_as(GetProfileResponseCollection, _response.json())  # type: ignore
        try:
            _response_json = _response.json()
        except JSONDecodeError:
            raise ApiError(status_code=_response.status_code, body=_response.text)
        raise ApiError(status_code=_response.status_code, body=_response_json)


class AsyncListsClient:
    def __init__(self, *, client_wrapper: AsyncClientWrapper):
        self._client_wrapper = client_wrapper

    async def get_lists(
        self,
        *,
        fields_list: typing.Union[
            typing.Optional[GetListsRequestFieldsListItem], typing.List[GetListsRequestFieldsListItem]
        ],
        fields_tag: typing.Union[
            typing.Optional[typing_extensions.Literal["name"]], typing.List[typing_extensions.Literal["name"]]
        ],
        filter: typing.Optional[str] = None,
        include: typing.Union[
            typing.Optional[typing_extensions.Literal["tags"]], typing.List[typing_extensions.Literal["tags"]]
        ],
        page_cursor: typing.Optional[str] = None,
    ) -> GetListListResponseCollectionCompoundDocument:
        """
        Get all lists in an account.

        Filter to request a subset of all lists. Lists can be filtered by `id`, `name`, `created`, and `updated` fields.

        Returns a maximum of 10 results per page.<br><br>*Rate limits*:<br>Burst: `75/s`<br>Steady: `700/m`

        **Scopes:**
        `list:read`

        Parameters:
            - fields_list: typing.Union[typing.Optional[GetListsRequestFieldsListItem], typing.List[GetListsRequestFieldsListItem]]. For more information please visit https://developers.klaviyo.com/en/v2023-08-15/reference/api-overview#sparse-fieldsets

            - fields_tag: typing.Union[typing.Optional[typing_extensions.Literal["name"]], typing.List[typing_extensions.Literal["name"]]]. For more information please visit https://developers.klaviyo.com/en/v2023-08-15/reference/api-overview#sparse-fieldsets

            - filter: typing.Optional[str]. For more information please visit https://developers.klaviyo.com/en/v2023-08-15/reference/api-overview#filtering<br>Allowed field(s)/operator(s):<br>`name`: `any`, `equals`<br>`id`: `any`, `equals`<br>`created`: `greater-than`<br>`updated`: `greater-than`

            - include: typing.Union[typing.Optional[typing_extensions.Literal["tags"]], typing.List[typing_extensions.Literal["tags"]]]. For more information please visit https://developers.klaviyo.com/en/v2023-08-15/reference/api-overview#relationships

            - page_cursor: typing.Optional[str]. For more information please visit https://developers.klaviyo.com/en/v2023-08-15/reference/api-overview#pagination
        """
        _response = await self._client_wrapper.httpx_client.request(
            "GET",
            urllib.parse.urljoin(f"{self._client_wrapper.get_base_url()}/", "api/lists"),
            params=remove_none_from_dict(
                {
                    "fields[list]": fields_list,
                    "fields[tag]": fields_tag,
                    "filter": filter,
                    "include": include,
                    "page[cursor]": page_cursor,
                }
            ),
            headers=self._client_wrapper.get_headers(),
            timeout=60,
        )
        if 200 <= _response.status_code < 300:
            return pydantic.parse_obj_as(GetListListResponseCollectionCompoundDocument, _response.json())  # type: ignore
        try:
            _response_json = _response.json()
        except JSONDecodeError:
            raise ApiError(status_code=_response.status_code, body=_response.text)
        raise ApiError(status_code=_response.status_code, body=_response_json)

    async def create_list(self, *, data: ListCreateQueryResourceObject) -> PostListCreateResponse:
        """
        Create a new list.<br><br>*Rate limits*:<br>Burst: `10/s`<br>Steady: `150/m`

        **Scopes:**
        `list:write`

        Parameters:
            - data: ListCreateQueryResourceObject.
        """
        _response = await self._client_wrapper.httpx_client.request(
            "POST",
            urllib.parse.urljoin(f"{self._client_wrapper.get_base_url()}/", "api/lists"),
            json=jsonable_encoder({"data": data}),
            headers=self._client_wrapper.get_headers(),
            timeout=60,
        )
        if 200 <= _response.status_code < 300:
            return pydantic.parse_obj_as(PostListCreateResponse, _response.json())  # type: ignore
        try:
            _response_json = _response.json()
        except JSONDecodeError:
            raise ApiError(status_code=_response.status_code, body=_response.text)
        raise ApiError(status_code=_response.status_code, body=_response_json)

    async def get_list(
        self,
        id: str,
        *,
        additional_fields_list: typing.Union[
            typing.Optional[typing_extensions.Literal["profile_count"]],
            typing.List[typing_extensions.Literal["profile_count"]],
        ],
        fields_list: typing.Union[
            typing.Optional[GetListRequestFieldsListItem], typing.List[GetListRequestFieldsListItem]
        ],
        fields_tag: typing.Union[
            typing.Optional[typing_extensions.Literal["name"]], typing.List[typing_extensions.Literal["name"]]
        ],
        include: typing.Union[
            typing.Optional[typing_extensions.Literal["tags"]], typing.List[typing_extensions.Literal["tags"]]
        ],
    ) -> GetListRetrieveResponseCompoundDocument:
        """
        Get a list with the given list ID.<br><br>*Rate limits*:<br>Burst: `75/s`<br>Steady: `700/m`<br><br>Rate limits when using the `additional-fields[list]=profile_count` parameter in your API request:<br>Burst: `1/s`<br>Steady: `15/m`<br><br>To learn more about how the `additional-fields` parameter impacts rate limits, check out our [Rate limits, status codes, and errors](https://developers.klaviyo.com/en/v2023-08-15/docs/rate_limits_and_error_handling) guide.

        **Scopes:**
        `list:read`

        Parameters:
            - id: str. Primary key that uniquely identifies this list. Generated by Klaviyo.

            - additional_fields_list: typing.Union[typing.Optional[typing_extensions.Literal["profile_count"]], typing.List[typing_extensions.Literal["profile_count"]]]. Request additional fields not included by default in the response. Supported values: 'profile_count'

            - fields_list: typing.Union[typing.Optional[GetListRequestFieldsListItem], typing.List[GetListRequestFieldsListItem]]. For more information please visit https://developers.klaviyo.com/en/v2023-08-15/reference/api-overview#sparse-fieldsets

            - fields_tag: typing.Union[typing.Optional[typing_extensions.Literal["name"]], typing.List[typing_extensions.Literal["name"]]]. For more information please visit https://developers.klaviyo.com/en/v2023-08-15/reference/api-overview#sparse-fieldsets

            - include: typing.Union[typing.Optional[typing_extensions.Literal["tags"]], typing.List[typing_extensions.Literal["tags"]]]. For more information please visit https://developers.klaviyo.com/en/v2023-08-15/reference/api-overview#relationships
        """
        _response = await self._client_wrapper.httpx_client.request(
            "GET",
            urllib.parse.urljoin(f"{self._client_wrapper.get_base_url()}/", f"api/lists/{id}"),
            params=remove_none_from_dict(
                {
                    "additional-fields[list]": additional_fields_list,
                    "fields[list]": fields_list,
                    "fields[tag]": fields_tag,
                    "include": include,
                }
            ),
            headers=self._client_wrapper.get_headers(),
            timeout=60,
        )
        if 200 <= _response.status_code < 300:
            return pydantic.parse_obj_as(GetListRetrieveResponseCompoundDocument, _response.json())  # type: ignore
        try:
            _response_json = _response.json()
        except JSONDecodeError:
            raise ApiError(status_code=_response.status_code, body=_response.text)
        raise ApiError(status_code=_response.status_code, body=_response_json)

    async def update_list(
        self, id: str, *, data: ListPartialUpdateQueryResourceObject
    ) -> PatchListPartialUpdateResponse:
        """
        Update the name of a list with the given list ID.<br><br>*Rate limits*:<br>Burst: `10/s`<br>Steady: `150/m`

        **Scopes:**
        `list:write`

        Parameters:
            - id: str. Primary key that uniquely identifies this list. Generated by Klaviyo.

            - data: ListPartialUpdateQueryResourceObject.
        """
        _response = await self._client_wrapper.httpx_client.request(
            "PATCH",
            urllib.parse.urljoin(f"{self._client_wrapper.get_base_url()}/", f"api/lists/{id}"),
            json=jsonable_encoder({"data": data}),
            headers=self._client_wrapper.get_headers(),
            timeout=60,
        )
        if 200 <= _response.status_code < 300:
            return pydantic.parse_obj_as(PatchListPartialUpdateResponse, _response.json())  # type: ignore
        try:
            _response_json = _response.json()
        except JSONDecodeError:
            raise ApiError(status_code=_response.status_code, body=_response.text)
        raise ApiError(status_code=_response.status_code, body=_response_json)

    async def delete_list(self, id: str) -> None:
        """
        Delete a list with the given list ID.<br><br>*Rate limits*:<br>Burst: `10/s`<br>Steady: `150/m`

        **Scopes:**
        `list:write`

        Parameters:
            - id: str. Primary key that uniquely identifies this list. Generated by Klaviyo.
        """
        _response = await self._client_wrapper.httpx_client.request(
            "DELETE",
            urllib.parse.urljoin(f"{self._client_wrapper.get_base_url()}/", f"api/lists/{id}"),
            headers=self._client_wrapper.get_headers(),
            timeout=60,
        )
        if 200 <= _response.status_code < 300:
            return
        try:
            _response_json = _response.json()
        except JSONDecodeError:
            raise ApiError(status_code=_response.status_code, body=_response.text)
        raise ApiError(status_code=_response.status_code, body=_response_json)

    async def get_list_relationships_tags(self, id: str) -> GetListTagRelationshipListResponseCollection:
        """
        Returns the tag IDs of all tags associated with the given list.<br><br>*Rate limits*:<br>Burst: `3/s`<br>Steady: `60/m`

        **Scopes:**
        `list:read`
        `tags:read`

        Parameters:
            - id: str.
        """
        _response = await self._client_wrapper.httpx_client.request(
            "GET",
            urllib.parse.urljoin(f"{self._client_wrapper.get_base_url()}/", f"api/lists/{id}/relationships/tags"),
            headers=self._client_wrapper.get_headers(),
            timeout=60,
        )
        if 200 <= _response.status_code < 300:
            return pydantic.parse_obj_as(GetListTagRelationshipListResponseCollection, _response.json())  # type: ignore
        try:
            _response_json = _response.json()
        except JSONDecodeError:
            raise ApiError(status_code=_response.status_code, body=_response.text)
        raise ApiError(status_code=_response.status_code, body=_response_json)

    async def get_list_tags(
        self,
        id: str,
        *,
        fields_tag: typing.Union[
            typing.Optional[typing_extensions.Literal["name"]], typing.List[typing_extensions.Literal["name"]]
        ],
    ) -> GetTagResponseCollection:
        """
        Return all tags associated with the given list ID.<br><br>*Rate limits*:<br>Burst: `3/s`<br>Steady: `60/m`

        **Scopes:**
        `list:read`
        `tags:read`

        Parameters:
            - id: str.

            - fields_tag: typing.Union[typing.Optional[typing_extensions.Literal["name"]], typing.List[typing_extensions.Literal["name"]]]. For more information please visit https://developers.klaviyo.com/en/v2023-08-15/reference/api-overview#sparse-fieldsets
        """
        _response = await self._client_wrapper.httpx_client.request(
            "GET",
            urllib.parse.urljoin(f"{self._client_wrapper.get_base_url()}/", f"api/lists/{id}/tags"),
            params=remove_none_from_dict({"fields[tag]": fields_tag}),
            headers=self._client_wrapper.get_headers(),
            timeout=60,
        )
        if 200 <= _response.status_code < 300:
            return pydantic.parse_obj_as(GetTagResponseCollection, _response.json())  # type: ignore
        try:
            _response_json = _response.json()
        except JSONDecodeError:
            raise ApiError(status_code=_response.status_code, body=_response.text)
        raise ApiError(status_code=_response.status_code, body=_response_json)

    async def get_list_relationships_profiles(
        self, id: str, *, page_cursor: typing.Optional[str] = None
    ) -> GetListRelationshipsResponseCollection:
        """
        Get profile membership [relationships](https://developers.klaviyo.com/en/reference/api_overview#relationships) for a list with the given list ID.<br><br>*Rate limits*:<br>Burst: `75/s`<br>Steady: `700/m`

        **Scopes:**
        `list:read`
        `profiles:read`

        Parameters:
            - id: str.

            - page_cursor: typing.Optional[str]. For more information please visit https://developers.klaviyo.com/en/v2023-08-15/reference/api-overview#pagination
        """
        _response = await self._client_wrapper.httpx_client.request(
            "GET",
            urllib.parse.urljoin(f"{self._client_wrapper.get_base_url()}/", f"api/lists/{id}/relationships/profiles"),
            params=remove_none_from_dict({"page[cursor]": page_cursor}),
            headers=self._client_wrapper.get_headers(),
            timeout=60,
        )
        if 200 <= _response.status_code < 300:
            return pydantic.parse_obj_as(GetListRelationshipsResponseCollection, _response.json())  # type: ignore
        try:
            _response_json = _response.json()
        except JSONDecodeError:
            raise ApiError(status_code=_response.status_code, body=_response.text)
        raise ApiError(status_code=_response.status_code, body=_response_json)

    async def create_list_relationships(self, id: str, *, data: typing.List[ListMembersAddQueryDataItem]) -> None:
        """
        Add a profile to a list with the given list ID.

        It is recommended that you use the [Subscribe Profiles endpoint](https://developers.klaviyo.com/en/reference/subscribe_profiles) if you're trying to give a profile consent to receive email marketing.

        This endpoint accepts a maximum of 1000 profiles per call.<br><br>*Rate limits*:<br>Burst: `10/s`<br>Steady: `150/m`

        **Scopes:**
        `list:write`
        `profiles:write`

        Parameters:
            - id: str.

            - data: typing.List[ListMembersAddQueryDataItem].
        """
        _response = await self._client_wrapper.httpx_client.request(
            "POST",
            urllib.parse.urljoin(f"{self._client_wrapper.get_base_url()}/", f"api/lists/{id}/relationships/profiles"),
            json=jsonable_encoder({"data": data}),
            headers=self._client_wrapper.get_headers(),
            timeout=60,
        )
        if 200 <= _response.status_code < 300:
            return
        try:
            _response_json = _response.json()
        except JSONDecodeError:
            raise ApiError(status_code=_response.status_code, body=_response.text)
        raise ApiError(status_code=_response.status_code, body=_response_json)

    async def delete_list_relationships(self, id: str, *, data: typing.List[ListMembersDeleteQueryDataItem]) -> None:
        """
        Remove a profile from a list with the given list ID.

        The provided profile will no longer receive marketing from this particular list once removed.

        Removing a profile from a list will not impact the profile's consent status or subscription status in general.
        To update a profile's subscription status, please use the [Unsubscribe Profiles endpoint](https://developers.klaviyo.com/en/reference/unsubscribe_profiles).

        This endpoint accepts a maximum of 1000 profiles per call.<br><br>*Rate limits*:<br>Burst: `10/s`<br>Steady: `150/m`

        **Scopes:**
        `list:write`
        `profiles:write`

        Parameters:
            - id: str.

            - data: typing.List[ListMembersDeleteQueryDataItem].
        """
        _response = await self._client_wrapper.httpx_client.request(
            "DELETE",
            urllib.parse.urljoin(f"{self._client_wrapper.get_base_url()}/", f"api/lists/{id}/relationships/profiles"),
            json=jsonable_encoder({"data": data}),
            headers=self._client_wrapper.get_headers(),
            timeout=60,
        )
        if 200 <= _response.status_code < 300:
            return
        try:
            _response_json = _response.json()
        except JSONDecodeError:
            raise ApiError(status_code=_response.status_code, body=_response.text)
        raise ApiError(status_code=_response.status_code, body=_response_json)

    async def get_list_profiles(
        self,
        id: str,
        *,
        additional_fields_profile: typing.Union[
            typing.Optional[typing_extensions.Literal["predictive_analytics"]],
            typing.List[typing_extensions.Literal["predictive_analytics"]],
        ],
        fields_profile: typing.Union[
            typing.Optional[GetListProfilesRequestFieldsProfileItem],
            typing.List[GetListProfilesRequestFieldsProfileItem],
        ],
        filter: typing.Optional[str] = None,
        page_cursor: typing.Optional[str] = None,
        page_size: typing.Optional[int] = None,
    ) -> GetProfileResponseCollection:
        """
        Get all profiles within a list with the given list ID.

        Filter to request a subset of all profiles. Profiles can be filtered by `email`, `phone_number`, and `push_token` fields.<br><br>*Rate limits*:<br>Burst: `75/s`<br>Steady: `700/m`<br><br>Rate limits when using the `additional-fields[profile]=predictive_analytics` parameter in your API request:<br>Burst: `10/s`<br>Steady: `150/m`<br><br>To learn more about how the `additional-fields` parameter impacts rate limits, check out our [Rate limits, status codes, and errors](https://developers.klaviyo.com/en/v2023-08-15/docs/rate_limits_and_error_handling) guide.

        **Scopes:**
        `list:read`
        `profiles:read`

        Parameters:
            - id: str.

            - additional_fields_profile: typing.Union[typing.Optional[typing_extensions.Literal["predictive_analytics"]], typing.List[typing_extensions.Literal["predictive_analytics"]]]. Request additional fields not included by default in the response. Supported values: 'predictive_analytics'

            - fields_profile: typing.Union[typing.Optional[GetListProfilesRequestFieldsProfileItem], typing.List[GetListProfilesRequestFieldsProfileItem]]. For more information please visit https://developers.klaviyo.com/en/v2023-08-15/reference/api-overview#sparse-fieldsets

            - filter: typing.Optional[str]. For more information please visit https://developers.klaviyo.com/en/v2023-08-15/reference/api-overview#filtering<br>Allowed field(s)/operator(s):<br>`email`: `any`<br>`phone_number`: `any`<br>`push_token`: `any`<br>`_kx`: `equals`

            - page_cursor: typing.Optional[str]. For more information please visit https://developers.klaviyo.com/en/v2023-08-15/reference/api-overview#pagination

            - page_size: typing.Optional[int]. Default: 20. Min: 1. Max: 100.
        """
        _response = await self._client_wrapper.httpx_client.request(
            "GET",
            urllib.parse.urljoin(f"{self._client_wrapper.get_base_url()}/", f"api/lists/{id}/profiles"),
            params=remove_none_from_dict(
                {
                    "additional-fields[profile]": additional_fields_profile,
                    "fields[profile]": fields_profile,
                    "filter": filter,
                    "page[cursor]": page_cursor,
                    "page[size]": page_size,
                }
            ),
            headers=self._client_wrapper.get_headers(),
            timeout=60,
        )
        if 200 <= _response.status_code < 300:
            return pydantic.parse_obj_as(GetProfileResponseCollection, _response.json())  # type: ignore
        try:
            _response_json = _response.json()
        except JSONDecodeError:
            raise ApiError(status_code=_response.status_code, body=_response.text)
        raise ApiError(status_code=_response.status_code, body=_response_json)
